@@ -1,5 +1,5 @@
 /*
- * $Id: comm_dbus.h 4895 2006-06-08 12:38:35Z guillem $
+ * $Id: comm_dbus.h 8466 2006-11-29 14:50:24Z guillem $
  *
  * Copyright (C) 2006 Nokia Corporation
  *
@@ -23,9 +23,9 @@
 #ifndef COMM_DBUS_H
 #define COMM_DBUS_H
 
-#define MAEMO_LAUNCHER_PATH			"/org/maemo/launcher"
-#define MAEMO_LAUNCHER_IFACE			"org.maemo.launcher"
-#define MAEMO_LAUNCHER_SIGNAL_APP_DIED		"ApplicationDied"
+#define LAUNCHER_PATH			"/org/maemo/launcher"
+#define LAUNCHER_IFACE			"org.maemo.launcher"
+#define LAUNCHER_SIGNAL_APP_DIED	"ApplicationDied"
 
 void comm_send_app_died(char *filename, int pid, int status);
 
