@@ -1,5 +1,5 @@
 /*
- * $Id: invoker.c 4895 2006-06-08 12:38:35Z guillem $
+ * $Id: invoker.c 8466 2006-11-29 14:50:24Z guillem $
  *
  * Copyright (C) 2005, 2006 Nokia Corporation
  *
@@ -358,7 +358,8 @@ main(int argc, char *argv[])
   {
     /* Called with a different name. Add the proper extension and launch it.
      * Do not try to parse any arguments. */
-    asprintf(&launch, "%s.launch", argv[0]);
+    if (asprintf(&launch, "%s.launch", argv[0]) < 0)
+      die(1, "allocating program name buffer");
     prog_name = launch;
     prog_argc = argc;
     prog_argv = argv;
