@@ -1,5 +1,5 @@
 /*
- * $Id: launcher.c 5059 2006-06-19 09:37:14Z guillem $
+ * $Id: launcher.c 8466 2006-11-29 14:50:24Z guillem $
  *
  * Copyright (C) 2005, 2006 Nokia Corporation
  *
@@ -23,6 +23,8 @@
 
 #define _GNU_SOURCE
 
+#include "config.h"
+
 #include <stdio.h>
 #include <stdint.h>
 #include <stdbool.h>
@@ -37,28 +39,17 @@
 #include <fcntl.h>
 #include <signal.h>
 #include <unistd.h>
-#include <dlfcn.h>
 #include <errno.h>
 
-#include "config.h"
-#include "ui.h"
 #include "report.h"
 #include "invokelib.h"
 #include "comm_dbus.h"
+#include "booster.h"
+#include "prog.h"
 
 /* FIXME: Should go into '/var/run/'. */
 #define LAUNCHER_PIDFILE "/tmp/"PROG_NAME".pid"
 
-typedef struct
-{
-  int options;
-  int argc;
-  char **argv;
-  char *filename;
-  char *name;
-  int prio;
-} prog_t;
-
 typedef struct
 {
   char *name;
@@ -79,16 +70,36 @@ static volatile bool sigchild_catched = false;
 static volatile bool sighup_catched = false;
 static bool send_app_died = false;
 
-#ifdef DEBUG
-extern char **environ;
-#endif
+#define OOM_ENABLE "0"
 
-typedef int (*entry_t)(int, char **);
+static void
+oom_unprotect(void)
+{
+  const char *filename = "/proc/self/oom_adj";
+  int fd;
+
+  fd = open(filename, O_WRONLY);
+  if (fd < 0)
+  {
+    error("opening file '%s'\n", filename);
+    return;
+  }
+
+  if (write(fd, OOM_ENABLE, strlen(OOM_ENABLE)) < 0)
+  {
+    if (errno == EPERM)
+      error("kernel w/o support for user processes rising the oom value\n");
+    else
+      error("could not write to file '%s'\n", filename);
+  }
+
+  close(fd);
+}
 
 static bool
-rise_oom_defense(int pid)
+rise_oom_defense(pid_t pid)
 {
-  int defender_pid;
+  pid_t defender_pid;
   int status;
 
   defender_pid = fork();
@@ -99,7 +110,7 @@ rise_oom_defense(int pid)
     char pidstr[20];
 
     snprintf(pidstr, sizeof(pidstr), "%d", pid);
-    execl(MAEMO_DEFENDER, MAEMO_DEFENDER, pidstr, NULL);
+    execl(DEFENDER, DEFENDER, pidstr, NULL);
     _exit(1);
   }
 
@@ -113,34 +124,11 @@ rise_oom_defense(int pid)
 }
 
 static void
-launch_process(prog_t *prog, ui_state state)
+launch_process(prog_t *prog)
 {
-  void *module;
-  entry_t entry;
-  char *error_s;
   int cur_prio;
 
-  /* Load the launched application. */
-  module = dlopen(prog->filename, RTLD_LAZY | RTLD_GLOBAL);
-  if (!module)
-  {
-    error("loading invoked application: '%s'\n", dlerror());
-    return;
-  }
-
-  dlerror();
-  entry = (entry_t)dlsym(module, "main");
-  error_s = dlerror();
-  if (error_s != NULL)
-  {
-    error("loading symbol 'main': '%s'\n", error_s);
-    return;
-  }
-
-  /* Initialize the launched application. */
-  setenv("_", prog->argv[0], true);
-
-  ui_client_init(prog->argv[0], state);
+  load_main(prog);
 
   /* Possibly restore process priority. */
   errno = 0;
@@ -148,42 +136,21 @@ launch_process(prog_t *prog, ui_state state)
   if (!errno && cur_prio < prog->prio)
     setpriority(PRIO_PROCESS, 0, prog->prio);
 
-  /* Protect our special childs from the oom monster. */
-  if (prog->prio < 0)
-    rise_oom_defense(getpid());
+  /* Unprotect our worthless childs from the oom monster. */
+  if (prog->prio >= 0)
+    oom_unprotect();
 
-#ifdef DEBUG
-  {
-    int i;
-
-    for (i = 0; environ[i] != NULL; i++)
-      debug("env[%i]='%s'\n", i, environ[i]);
-
-    for (i = 0; i < prog->argc; i++)
-      debug("argv[%i]='%s'\n", i, prog->argv[i]);
-  }
+  print_prog_env_argv(prog);
 
   debug("launching process: '%s'\n", prog->filename);
 
+#ifdef DEBUG
   report_set_output(report_console);
 #else
   report_set_output(report_none);
 #endif
 
-  _exit(entry(prog->argc, prog->argv));
-}
-
-static void
-set_progname(char *progname, int argc, char **argv)
-{
-  int argvlen = 0;
-  int i;
-
-  for (i = 0; i < argc; i++)
-    argvlen += strlen(argv[i]) + 1;
-
-  memset(argv[0], 0, argvlen);
-  strncpy(argv[0], progname, argvlen - 1);
+  _exit(prog->entry(prog->argc, prog->argv));
 }
 
 static int
@@ -326,7 +293,7 @@ invoked_get_prio(int fd, prog_t *prog)
 }
 
 static bool
-invoked_send_pid(int fd, int pid)
+invoked_send_pid(int fd, pid_t pid)
 {
   invoke_send_msg(fd, INVOKER_MSG_PID);
   invoke_send_msg(fd, pid);
@@ -434,6 +401,25 @@ grow_childs(kindergarten_t *childs)
   return true;
 }
 
+static bool
+release_childs(kindergarten_t *childs)
+{
+  int i;
+  child_t *list = childs->list;
+
+  for (i = 0; i < childs->used; i++)
+  {
+    close(list[i].sock);
+    free(list[i].name);
+  }
+
+  childs->used = 0;
+  childs->n = 0;
+  free(childs->list);
+
+  return true;
+}
+
 static int
 get_child_slot_by_pid(kindergarten_t *childs, pid_t pid)
 {
@@ -532,7 +518,6 @@ static void
 clean_childs(kindergarten_t *childs)
 {
   int status;
-  char *cause;
   pid_t childpid;
 
   while ((childpid = waitpid(-1, &status, WNOHANG)) > 0)
@@ -540,13 +525,13 @@ clean_childs(kindergarten_t *childs)
     release_child_slot(childs, childpid, status);
 
     if (WIFEXITED(status))
-      asprintf(&cause, "exit()=%d", WEXITSTATUS(status));
+      info("child (pid=%d) terminated due to exit()=%d\n", childpid,
+	   WEXITSTATUS(status));
     else if (WIFSIGNALED(status))
-      asprintf(&cause, "signal=%d", WTERMSIG(status));
-
-    info("child (pid=%d) exited due to %s\n", childpid, cause);
-
-    free(cause);
+      info("child (pid=%d) terminated due to signal=%d\n", childpid,
+	   WTERMSIG(status));
+    else
+      info("child (pid=%d) terminated due to unknown status=%d\n", status);
   }
 }
 
@@ -600,6 +585,12 @@ sigs_restore(void)
   sigaction(SIGHUP, &sig, NULL);
 }
 
+static void
+sigs_interrupt(int flag)
+{
+  siginterrupt(SIGCHLD, flag);
+}
+
 static void
 env_init(void)
 {
@@ -673,6 +664,7 @@ usage(int status)
 	 "Options:\n"
 	 "  --daemon            Fork and go into the background.\n"
 	 "  --pidfile FILE      Specify a different pid file (default %s).\n"
+	 "  --booster MODULE    Specify a booster module to use (default '%s').\n"
 	 "  --send-app-died     Send application died signal.\n"
 	 "  --quiet             Do not print anything.\n"
 	 "  --version           Print program version.\n"
@@ -681,7 +673,7 @@ usage(int status)
 	 "Use the invoker to start a <shared object> from the launcher.\n"
 	 "Where <shared object> is a binary including a 'main' symbol.\n"
 	 "Note that the binary needs to be linked with -shared or -pie.\n",
-	 PROG_NAME, LAUNCHER_PIDFILE);
+	 PROG_NAME, LAUNCHER_PIDFILE, BOOSTER_DEFAULT);
 
   exit(status);
 }
@@ -689,7 +681,7 @@ usage(int status)
 int
 main(int argc, char *argv[])
 {
-  ui_state state;
+  booster_t booster = { .name = BOOSTER_DEFAULT };
   kindergarten_t *childs;
   const int initial_child_slots = 64;
   int i;
@@ -711,6 +703,11 @@ main(int argc, char *argv[])
       if (argv[++i])
 	pidfilename = argv[i];
     }
+    else if (strcmp(argv[i], "--booster") == 0)
+    {
+      if (argv[++i])
+	booster.name = argv[i];
+    }
     else if (strcmp(argv[i], "--send-app-died") == 0)
       send_app_died = true;
     else if (strcmp(argv[i], "--version") == 0)
@@ -724,7 +721,9 @@ main(int argc, char *argv[])
   /*
    * Daemon initialization.
    */
-  state = ui_daemon_init(&argc, &argv);
+  booster_module_load(&booster);
+
+  booster.state = booster.api->booster_preinit(&argc, &argv);
 
   sigs_init();
   env_init();
@@ -753,10 +752,10 @@ main(int argc, char *argv[])
     int errno_accept;
 
     /* Accept a new invokation. */
-    siginterrupt(SIGCHLD, 1);
+    sigs_interrupt(1);
     sd = accept(fd, NULL, NULL);
     errno_accept = errno;
-    siginterrupt(SIGCHLD, 0);
+    sigs_interrupt(0);
 
     /* Handle signals received. */
     if (sigchild_catched)
@@ -767,7 +766,7 @@ main(int argc, char *argv[])
 
     if (sighup_catched)
     {
-      ui_state_reload(state);
+      booster.api->booster_reload(booster.state);
       sighup_catched = false;
     }
 
@@ -801,6 +800,9 @@ main(int argc, char *argv[])
       break;
 
     case 0: /* Child. */
+      if (setsid() < 0)
+	error("setting session id\n");
+
       sigs_restore();
 
       invoked_get_actions(sd, &prog);
@@ -809,12 +811,17 @@ main(int argc, char *argv[])
       close(fd);
       free(prog.name);
 
+      release_childs(childs);
+
       /* Invoke it. */
       if (prog.filename)
       {
         info("invoking '%s'\n", prog.filename);
         set_progname(prog.argv[0], argc, argv);
-        launch_process(&prog, state);
+
+	booster.api->booster_init(prog.argv[0], booster.state);
+
+	launch_process(&prog);
       }
       else
         error("nothing to invoke\n");
