@@ -1,14 +1,13 @@
 /*
- * report.h
+ * $Id: report.h 8466 2006-11-29 14:50:24Z guillem $
  *
- * Copyright (C) 2005 Nokia
+ * Copyright (C) 2005, 2006 Nokia Corporation
  *
  * Author: Guillem Jover <guillem.jover@nokia.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,23 +23,38 @@
 #ifndef REPORT_H
 #define REPORT_H
 
+#ifdef __GNUC__
+#define ATTR_NORET __attribute__((noreturn))
+#else
+#define ATTR_NORET
+#endif
+
 enum report_output {
   report_console,
   report_syslog,
   report_none
 };
 
+enum report_type {
+  report_debug,
+  report_info,
+  report_error,
+  report_fatal
+};
+
 extern void report_set_output(enum report_output new_output);
+extern void report(enum report_type type, char *msg, ...);
 
 #ifdef DEBUG
-extern void debug(char *msg, ...);
+#define debug(msg, ...) report(report_debug, msg, ##__VA_ARGS__)
 #else
 #define debug(...)
 #endif
 
-extern void info(char *msg, ...);
-extern void error(char *msg, ...);
-extern void __attribute__((noreturn)) die(int status, char *msg, ...);
+#define info(msg, ...) report(report_info, msg, ##__VA_ARGS__)
+#define error(msg, ...) report(report_error, msg, ##__VA_ARGS__)
+
+extern void ATTR_NORET die(int status, char *msg, ...);
 
 #endif
 
